from copy import deepcopy
from fontTools.ttLib import TTFont
from fontTools.misc.testTools import getXML
from fontTools.otlLib.builder import buildStatTable
from fontTools.varLib.instancer.names import _updateUniqueIdNameRecord, NameID
from fontTools.ttLib.tables._f_v_a_r import NamedInstance
from pkg_resources import resource_filename
from google.protobuf import text_format
from collections import OrderedDict
from axisregistry.axes_pb2 import AxisProto
from collections import defaultdict
from itertools import chain
import logging
from glob import glob
import os

try:
    from ._version import version as __version__  # type: ignore
except ImportError:
    __version__ = "0.0.0+unknown"

log = logging.getLogger(__file__)

# TODO we may have more of these. Please note that some applications may not
# implement variable font style linking.
LINKED_VALUES = {
    "wght": {400.0: 700.0},
    "ital": {0.0: 1.0},
}

# Static font styles. The GF api only support the following static font styles
GF_STATIC_STYLES = OrderedDict(
    [
        ("Thin", 100),
        ("ExtraLight", 200),
        ("Light", 300),
        ("Regular", 400),
        ("Medium", 500),
        ("SemiBold", 600),
        ("Bold", 700),
        ("ExtraBold", 800),
        ("Black", 900),
        ("Thin Italic", 100),
        ("ExtraLight Italic", 200),
        ("Light Italic", 300),
        ("Italic", 400),
        ("Medium Italic", 500),
        ("SemiBold Italic", 600),
        ("Bold Italic", 700),
        ("ExtraBold Italic", 800),
        ("Black Italic", 900),
    ]
)


def load_protobuf(klass, path):
    message = klass()
    with open(path, "rb") as text_data:
        text_format.Merge(text_data.read(), message)
    return message


class AxisRegistry:
    def __init__(self, fp=resource_filename("axisregistry", "data")):
        axis_fps = [fp for fp in glob(os.path.join(fp, "*.textproto"))]
        self._data = {}
        for fp in axis_fps:
            axis = load_protobuf(AxisProto, fp)
            self._data[axis.tag] = axis

    def __getitem__(self, k):
        return self._data[k]

    def __iter__(self):
        for i in self._data.keys():
            yield i

    def keys(self):
        return self._data.keys()

    def items(self):
        return self._data.items()

    def get_fallback(self, name):
        for a in self:
            for fallback in self[a].fallback:
                if name == fallback.name:
                    return a, fallback
        return None, None

    def fallbacks_in_fvar(self, ttFont):
        res = defaultdict(list)
        axes_in_font = {
            a.axisTag: {"min": a.minValue, "max": a.maxValue}
            for a in ttFont["fvar"].axes
        }
        for axis in axes_in_font:
            if axis not in self.keys():
                log.warn(f"Axis {axis} not found in GF Axis Registry!")
                continue
            for fallback in self[axis].fallback:
                if (
                    fallback.value < axes_in_font[axis]["min"]
                    or fallback.value > axes_in_font[axis]["max"]
                ):
                    continue
                res[axis].append(fallback)
        return res

    def fallbacks_in_name_table(self, ttFont):
        res = []
        name_table = ttFont["name"]
        tokens = (
            name_table.getBestFamilyName().split()[1:]
            + name_table.getBestSubFamilyName().split()
        )
        fvar_axes_in_font = [a.axisTag for a in ttFont["fvar"].axes]
        for token in tokens:
            axis, fallback = axis_registry.get_fallback(token)
            if any([not axis, axis in fvar_axes_in_font, fallback in res]):
                continue
            res.append((axis, fallback))
        return res

    def fallback_for_value(self, axis_tag, value):
        if axis_tag in axis_registry:
            return next(
                (f for f in axis_registry[axis_tag].fallback if f.value == value),
                None,
            )
        return None


axis_registry = AxisRegistry()
# sort user axes by alphabetical order and append presorted registered axes
AXIS_ORDER = sorted([i for i in axis_registry if i.isupper()]) + [
    "opsz",
    "wdth",
    "wght",
    "ital",
    "slnt",
]


def is_variable(ttFont):
    return "fvar" in ttFont


def _fvar_dflts(ttFont):
    res = OrderedDict()
    for a in ttFont["fvar"].axes:
        fallback = axis_registry.fallback_for_value(a.axisTag, a.defaultValue)
        if fallback:
            name = fallback.name
            elided = fallback.value == axis_registry[
                a.axisTag
            ].default_value and name not in ["Regular", "Italic"]
        else:
            name = None
            elided = True  # since we can't find a name for it, keep it elided
        res[a.axisTag] = {"value": a.defaultValue, "name": name, "elided": elided}
    return res


def build_stat(ttFont, sibling_ttFonts=[]):
    log.info("Building STAT table")
    assert is_variable(ttFont), "not a VF!"
    fallbacks_in_fvar = axis_registry.fallbacks_in_fvar(ttFont)
    fallbacks_in_siblings = list(
        chain.from_iterable(
            axis_registry.fallbacks_in_name_table(f) for f in sibling_ttFonts
        )
    )
    fallbacks_in_names = axis_registry.fallbacks_in_name_table(ttFont)
    nametable = ttFont["name"]
    fvar = ttFont["fvar"]

    # rm old STAT table and associated name table records
    fvar_nameids = set(i.subfamilyNameID for i in fvar.instances)
    if "STAT" in ttFont:
        stat = ttFont["STAT"]
        if stat.table.AxisValueCount > 0:
            axis_values = stat.table.AxisValueArray.AxisValue
            for ax in axis_values:
                if ax.ValueNameID not in fvar_nameids:
                    nametable.removeNames(nameID=ax.ValueNameID)
        if stat.table.DesignAxisCount > 0:
            axes = stat.table.DesignAxisRecord.Axis
            for ax in axes:
                if ax.AxisNameID not in fvar_nameids:
                    nametable.removeNames(nameID=ax.AxisNameID)
        del ttFont["STAT"]

    res = []
    # use fontTools build_stat. Link contains function params and usage example
    # https://github.com/fonttools/fonttools/blob/a293606fc8c88af8510d0688a6a36271ff4ff350/Lib/fontTools/otlLib/builder.py#L2683
    seen_axes = set()
    for axis, fallbacks in fallbacks_in_fvar.items():
        seen_axes.add(axis)
        a = {"tag": axis, "name": axis_registry[axis].display_name, "values": []}
        for fallback in fallbacks:
            a["values"].append(
                {
                    "name": fallback.name,
                    "value": fallback.value,
                    # include flags and linked values
                    "flags": 0x2
                    if fallback.value == axis_registry[axis].default_value
                    else 0x0,
                }
            )
            if axis in LINKED_VALUES and fallback.value in LINKED_VALUES[axis]:
                a["values"][-1]["linkedValue"] = LINKED_VALUES[axis][fallback.value]
        res.append(a)

    for axis, fallback in fallbacks_in_names:
        if axis in seen_axes:
            continue
        a = {
            "tag": axis,
            "name": axis_registry[axis].display_name,
            "values": [{"name": fallback.name, "value": fallback.value, "flags": 0x0}],
        }
        if axis in LINKED_VALUES and fallback.value in LINKED_VALUES[axis]:
            a["values"][0]["linkedValue"] = LINKED_VALUES[axis][fallback.value]
        res.append(a)

    for axis, fallback in fallbacks_in_siblings:
        if axis in seen_axes:
            continue
        value = 0.0
        a = {
            "tag": axis,
            "name": axis_registry[axis].display_name,
            "values": [{"name": "Normal", "value": value, "flags": 0x2}],
        }
        if axis in LINKED_VALUES and value in LINKED_VALUES[axis]:
            a["values"][0]["linkedValue"] = LINKED_VALUES[axis][value]
        res.append(a)
    buildStatTable(ttFont, res, macNames=False)


def build_name_table(ttFont, family_name=None, style_name=None, siblings=[]):
    log.info("Building name table")
    name_table = ttFont["name"]
    family_name = family_name if family_name else name_table.getBestFamilyName()
    style_name = style_name if style_name else name_table.getBestSubFamilyName()
    if is_variable(ttFont):
        return build_vf_name_table(ttFont, family_name, siblings=siblings)
    return build_static_name_table_v1(ttFont, family_name, style_name)


def _fvar_instance_collisions(ttFont, siblings=[]):
    """Check if a font family is going to have colliding fvar instances.

    Collision occur when a family has has 2+ roman styles or 2+ italic
    styles."""

    def is_italic(font):
        return font["post"].italicAngle != 0.0

    family_styles = [is_italic(f) for f in siblings + [ttFont]]

    return len(family_styles) != len(set(family_styles))


def build_vf_name_table(ttFont, family_name, siblings=[]):
    # VF name table should reflect the 0 origin of the font!
    assert is_variable(ttFont), "Not a VF!"
    style_name = _vf_style_name(ttFont, family_name)
    if _fvar_instance_collisions(ttFont, siblings):
        build_static_name_table_v1(ttFont, family_name, style_name)
    else:
        build_static_name_table(ttFont, family_name, style_name)
    build_variations_ps_name(ttFont, family_name)


def build_variations_ps_name(ttFont, family_name=None):
    assert is_variable(ttFont), "Not a VF!"
    if not family_name:
        family_name = ttFont["name"].getBestFamilyName()
    font_styles = axis_registry.fallbacks_in_name_table(ttFont)
    if font_styles:
        vf_ps = family_name.replace(" ", "") + "".join(
            [
                fallback.name
                for _, fallback in font_styles
                if fallback.name not in family_name
            ]
        )
    else:
        vf_ps = family_name.replace(" ", "")
    ttFont["name"].setName(vf_ps, NameID.VARIATIONS_POSTSCRIPT_NAME_PREFIX, 3, 1, 0x409)


def _vf_style_name(ttFont, family_name):
    fvar_dflts = _fvar_dflts(ttFont)
    res = []
    for axis_name in AXIS_ORDER:
        if axis_name not in fvar_dflts:
            continue
        value = fvar_dflts[axis_name]
        if not value["elided"]:
            res.append(value["name"])

    family_name_tokens = family_name.split()
    font_styles = axis_registry.fallbacks_in_name_table(ttFont)
    for _, fallback in font_styles:
        if fallback.name not in res and fallback.name not in family_name_tokens:
            res.append(fallback.name)

    name = " ".join(res).replace("Regular Italic", "Italic")
    log.debug(f"Built VF style name: '{name}'")
    return name


def build_fvar_instances(ttFont, axis_dflts={}):
    """Replace a variable font's fvar instances with a set of new instances
    which conform to the Google Fonts instance spec:
    https://github.com/googlefonts/gf-docs/tree/main/Spec#fvar-instances
    """
    assert is_variable(ttFont), "Not a VF!"
    log.info("Building fvar instances")
    fvar = ttFont["fvar"]
    name_table = ttFont["name"]
    style_name = name_table.getBestSubFamilyName()

    # Protect name IDs which are shared with the STAT table
    stat_nameids = []
    if "STAT" in ttFont:
        if ttFont["STAT"].table.AxisValueCount > 0:
            stat_nameids = [
                av.ValueNameID for av in ttFont["STAT"].table.AxisValueArray.AxisValue
            ]

    # rm old fvar subfamily and ps name records
    for inst in fvar.instances:
        if inst.subfamilyNameID not in [2, 17] + stat_nameids:
            name_table.removeNames(nameID=inst.subfamilyNameID)
        if inst.postscriptNameID not in [65535, 6]:
            name_table.removeNames(nameID=inst.postscriptNameID)

    fvar_dflts = _fvar_dflts(ttFont)
    if not axis_dflts:
        axis_dflts = {k: v["value"] for k, v in fvar_dflts.items()}

    is_italic = "Italic" in style_name
    is_roman_and_italic = any(a for a in ("slnt", "ital") if a in fvar_dflts)

    fallbacks = axis_registry.fallbacks_in_fvar(ttFont)
    # some families may not have a wght axis e.g
    # https://fonts.google.com/specimen/League+Gothic
    # these families just have a single weight which is Regular
    if "wght" not in fvar_dflts:
        fallback = next(
            (f for f in axis_registry["wght"].fallback if f.value == 400.0), None
        )
        fallbacks["wght"] = [fallback]

    wght_fallbacks = fallbacks["wght"]

    ital_axis = next((a for a in fvar.axes if a.axisTag == "ital"), None)
    slnt_axis = next((a for a in fvar.axes if a.axisTag == "slnt"), None)

    def gen_instances(is_italic):
        results = []
        for fallback in wght_fallbacks:
            name = fallback.name if not is_italic else f"{fallback.name} Italic".strip()
            name = name.replace("Regular Italic", "Italic")

            coordinates = {k: v for k, v in axis_dflts.items()}
            if "wght" in fvar_dflts:
                coordinates["wght"] = fallback.value
            if is_italic:
                if ital_axis:
                    coordinates["ital"] = ital_axis.minValue
                elif slnt_axis:
                    coordinates["slnt"] = slnt_axis.minValue

            inst = NamedInstance()
            inst.subfamilyNameID = name_table.addName(name)
            inst.coordinates = coordinates
            log.debug(f"Adding fvar instance: {name}: {coordinates}")
            results.append(inst)
        return results

    instances = []
    if is_roman_and_italic:
        for bool_ in (False, True):
            instances += gen_instances(is_italic=bool_)
    elif is_italic:
        instances += gen_instances(is_italic=True)
    else:
        instances += gen_instances(is_italic=False)
    fvar.instances = instances


def build_static_name_table(ttFont, family_name, style_name):
    # stip mac names
    name_table = ttFont["name"]
    name_table.removeNames(platformID=1)
    existing_name = ttFont["name"].getBestFamilyName()

    names = {}
    is_ribbi = (
        True if style_name in ("Regular", "Italic", "Bold", "Bold Italic") else False
    )
    if is_ribbi:
        full_name = f"{family_name} {style_name}"
        ps_name = f"{family_name}-{style_name}".replace(" ", "")
        names[(NameID.FAMILY_NAME, 3, 1, 0x409)] = family_name
        names[(NameID.SUBFAMILY_NAME, 3, 1, 0x409)] = style_name
        names[(NameID.FULL_FONT_NAME, 3, 1, 0x409)] = full_name
        names[(NameID.POSTSCRIPT_NAME, 3, 1, 0x409)] = ps_name
        for name_id in (
            NameID.TYPOGRAPHIC_FAMILY_NAME,
            NameID.TYPOGRAPHIC_SUBFAMILY_NAME,
            21,
            22,
        ):
            name_table.removeNames(nameID=name_id)
    else:
        style_tokens = style_name.split()
        new_family_name = family_name.split()
        is_italic = "Italic" in style_tokens
        for t in style_tokens:
            if t in ["Regular", "Italic"] or t in new_family_name:
                continue
            new_family_name.append(t)
        new_family_name = " ".join(new_family_name)
        new_style_name = "Italic" if is_italic else "Regular"
        full_name = f"{family_name} {style_name}"
        ps_name = f"{family_name}-{style_name}".replace(" ", "")

        names[(NameID.FAMILY_NAME, 3, 1, 0x409)] = new_family_name
        names[(NameID.SUBFAMILY_NAME, 3, 1, 0x409)] = new_style_name
        names[(NameID.FULL_FONT_NAME, 3, 1, 0x409)] = full_name
        names[(NameID.POSTSCRIPT_NAME, 3, 1, 0x409)] = ps_name
        names[(NameID.TYPOGRAPHIC_FAMILY_NAME, 3, 1, 0x409)] = family_name
        names[(NameID.TYPOGRAPHIC_SUBFAMILY_NAME, 3, 1, 0x409)] = style_name
        # we do not use WWS names since we use the RIBBI naming schema
        for name_id in (21, 22):
            name_table.removeNames(nameID=name_id)

    names[(NameID.UNIQUE_FONT_IDENTIFIER, 3, 1, 0x409)] = _updateUniqueIdNameRecord(
        ttFont, {k[0]: v for k, v in names.items()}, (3, 1, 0x409)
    )
    for k, v in names.items():
        log.debug(f"Adding name record {k}: {v}")
        name_table.setName(v, *k)

    # Replace occurences of old family name in untouched records
    skip_ids = [i.numerator for i in NameID]
    for r in ttFont["name"].names:
        if r.nameID in skip_ids:
            continue
        current = r.toUnicode()
        if existing_name not in current:
            continue
        if " " not in current:
            replacement = current.replace(existing_name, family_name).replace(" ", "")
        else:
            replacement = current.replace(existing_name, family_name)
        ttFont["name"].setName(
            replacement, r.nameID, r.platformID, r.platEncID, r.langID
        )


def build_static_name_table_v1(ttFont, family_name, style_name):
    """Pre VF name tables, this version can only accept wght + ital"""
    non_weight_tokens = []
    v1_tokens = []
    tokens = style_name.split()
    for t in tokens:
        if t not in GF_STATIC_STYLES:
            non_weight_tokens.append(t)
        else:
            v1_tokens.append(t)

    family_tokens = family_name.split()
    new_family_name = []
    for t in family_tokens:
        if t in non_weight_tokens or t in new_family_name:
            continue
        new_family_name.append(t)
    for t in non_weight_tokens:
        new_family_name.append(t)

    family_name = " ".join(new_family_name)
    style_name = " ".join(v1_tokens).replace("Regular Italic", "Italic").strip()
    style_name = style_name or "Regular"
    log.debug(f"New family name: {family_name}")
    log.debug(f"New style name: {style_name}")
    build_static_name_table(ttFont, family_name, style_name)


def build_filename(ttFont):
    name_table = ttFont["name"]
    family_name = name_table.getBestFamilyName()
    style_name = name_table.getBestSubFamilyName()
    _, ext = os.path.splitext(ttFont.reader.file.name)
    if is_variable(ttFont):
        is_italic = "Italic" in style_name
        axes = _fvar_dflts(ttFont).keys()
        axes = sorted([a for a in axes if a.isupper()]) + sorted(
            [a for a in axes if a.islower()]
        )
        if is_italic:
            return f"{family_name}-Italic[{','.join(axes)}]{ext}".replace(" ", "")
        return f"{family_name}[{','.join(axes)}]{ext}".replace(" ", "")
    return f"{family_name}-{style_name}{ext}".replace(" ", "")


def dump(table, ttFont=None):
    return "\n".join(getXML(table.toXML, ttFont))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: axes.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="axes.proto",
    package="",
    syntax="proto2",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\naxes.proto"\xe6\x01\n\tAxisProto\x12\x0b\n\x03tag\x18\x01 \x01(\t\x12\x14\n\x0c\x64isplay_name\x18\x07 \x01(\t\x12\x11\n\tmin_value\x18\x02 \x01(\x02\x12\x15\n\rdefault_value\x18\x03 \x01(\x02\x12\x11\n\tmax_value\x18\x04 \x01(\x02\x12\x11\n\tprecision\x18\x05 \x01(\x05\x12 \n\x08\x66\x61llback\x18\x06 \x03(\x0b\x32\x0e.FallbackProto\x12\x13\n\x0b\x64\x65scription\x18\x08 \x01(\t\x12\x15\n\rfallback_only\x18\t \x01(\x08\x12\x18\n\x10illustration_url\x18\n \x01(\t"B\n\rFallbackProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02\x12\x14\n\x0c\x64isplay_name\x18\x03 \x01(\t',
)


_AXISPROTO = _descriptor.Descriptor(
    name="AxisProto",
    full_name="AxisProto",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="tag",
            full_name="AxisProto.tag",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="display_name",
            full_name="AxisProto.display_name",
            index=1,
            number=7,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="min_value",
            full_name="AxisProto.min_value",
            index=2,
            number=2,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="default_value",
            full_name="AxisProto.default_value",
            index=3,
            number=3,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="max_value",
            full_name="AxisProto.max_value",
            index=4,
            number=4,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="precision",
            full_name="AxisProto.precision",
            index=5,
            number=5,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="fallback",
            full_name="AxisProto.fallback",
            index=6,
            number=6,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="description",
            full_name="AxisProto.description",
            index=7,
            number=8,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="fallback_only",
            full_name="AxisProto.fallback_only",
            index=8,
            number=9,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="illustration_url",
            full_name="AxisProto.illustration_url",
            index=9,
            number=10,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto2",
    extension_ranges=[],
    oneofs=[],
    serialized_start=15,
    serialized_end=245,
)


_FALLBACKPROTO = _descriptor.Descriptor(
    name="FallbackProto",
    full_name="FallbackProto",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="name",
            full_name="FallbackProto.name",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="FallbackProto.value",
            index=1,
            number=2,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="display_name",
            full_name="FallbackProto.display_name",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto2",
    extension_ranges=[],
    oneofs=[],
    serialized_start=247,
    serialized_end=313,
)

_AXISPROTO.fields_by_name["fallback"].message_type = _FALLBACKPROTO
DESCRIPTOR.message_types_by_name["AxisProto"] = _AXISPROTO
DESCRIPTOR.message_types_by_name["FallbackProto"] = _FALLBACKPROTO
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

AxisProto = _reflection.GeneratedProtocolMessageType(
    "AxisProto",
    (_message.Message,),
    {
        "DESCRIPTOR": _AXISPROTO,
        "__module__": "axes_pb2"
        # @@protoc_insertion_point(class_scope:AxisProto)
    },
)
_sym_db.RegisterMessage(AxisProto)

FallbackProto = _reflection.GeneratedProtocolMessageType(
    "FallbackProto",
    (_message.Message,),
    {
        "DESCRIPTOR": _FALLBACKPROTO,
        "__module__": "axes_pb2"
        # @@protoc_insertion_point(class_scope:FallbackProto)
    },
)
_sym_db.RegisterMessage(FallbackProto)


# @@protoc_insertion_point(module_scope)



from fontTools.ttLib import TTFont
from fontTools.misc.testTools import getXML
from axisregistry import (
    build_name_table,
    build_filename,
    build_fvar_instances,
    build_stat,
    build_variations_ps_name,
    _fvar_dflts,
    _fvar_instance_collisions,
)
import pytest
import os

CWD = os.path.dirname(__file__)
DATA_DIR = os.path.join(CWD, "data")

roboto_flex_fp = os.path.join(
    DATA_DIR,
    "RobotoFlex[GRAD,XOPQ,XTRA,YOPQ,YTAS,YTDE,YTFI,YTLC,YTUC,opsz,slnt,wdth,wght].ttf",
)
mavenpro_fp = os.path.join(DATA_DIR, "MavenPro-Regular.ttf")
opensans_roman_fp = os.path.join(DATA_DIR, "OpenSans[wdth,wght].ttf")
opensans_italic_fp = os.path.join(DATA_DIR, "OpenSans-Italic[wdth,wght].ttf")
opensans_cond_roman_fp = os.path.join(DATA_DIR, "OpenSansCondensed[wght].ttf")
opensans_cond_italic_fp = os.path.join(DATA_DIR, "OpenSansCondensed-Italic[wght].ttf")
wonky_fp = os.path.join(DATA_DIR, "Wonky[wdth,wght].ttf")


@pytest.fixture
def static_font():
    return TTFont(mavenpro_fp)


def _test_names(ttFont, expected):
    name_table = ttFont["name"]
    for k, v in expected.items():
        record = name_table.getName(*k)
        if not record:
            assert record == v, f"{k} record is incorrect, expected {v} got {record}"
        else:
            record_string = record.toUnicode()
            assert (
                record_string == v
            ), f"{k} record is incorrect, expected {v} got {record}"


@pytest.mark.parametrize(
    "fp, family_name, style_name, siblings, expected",
    [
        # Maven Pro Regular
        (
            mavenpro_fp,
            "Maven Pro",
            "Regular",
            [],
            {
                (1, 3, 1, 0x409): "Maven Pro",
                (2, 3, 1, 0x409): "Regular",
                (3, 3, 1, 0x409): "2.003;NONE;MavenPro-Regular",
                (4, 3, 1, 0x409): "Maven Pro Regular",
                (6, 3, 1, 0x409): "MavenPro-Regular",
                (16, 3, 1, 0x409): None,
                (17, 3, 1, 0x409): None,
            },
        ),
        # Maven Pro Italic
        (
            mavenpro_fp,
            "Maven Pro",
            "Italic",
            [],
            {
                (1, 3, 1, 0x409): "Maven Pro",
                (2, 3, 1, 0x409): "Italic",
                (3, 3, 1, 0x409): "2.003;NONE;MavenPro-Italic",
                (4, 3, 1, 0x409): "Maven Pro Italic",
                (6, 3, 1, 0x409): "MavenPro-Italic",
                (16, 3, 1, 0x409): None,
                (17, 3, 1, 0x409): None,
            },
        ),
        # Maven Pro Bold
        (
            mavenpro_fp,
            "Maven Pro",
            "Bold",
            [],
            {
                (1, 3, 1, 0x409): "Maven Pro",
                (2, 3, 1, 0x409): "Bold",
                (3, 3, 1, 0x409): "2.003;NONE;MavenPro-Bold",
                (4, 3, 1, 0x409): "Maven Pro Bold",
                (6, 3, 1, 0x409): "MavenPro-Bold",
                (16, 3, 1, 0x409): None,
                (17, 3, 1, 0x409): None,
            },
        ),
        # Maven Pro Bold Italic
        (
            mavenpro_fp,
            "Maven Pro",
            "Bold Italic",
            [],
            {
                (1, 3, 1, 0x409): "Maven Pro",
                (2, 3, 1, 0x409): "Bold Italic",
                (3, 3, 1, 0x409): "2.003;NONE;MavenPro-BoldItalic",
                (4, 3, 1, 0x409): "Maven Pro Bold Italic",
                (6, 3, 1, 0x409): "MavenPro-BoldItalic",
                (16, 3, 1, 0x409): None,
                (17, 3, 1, 0x409): None,
            },
        ),
        # Maven Pro Black
        (
            mavenpro_fp,
            "Maven Pro",
            "Black",
            [],
            {
                (1, 3, 1, 0x409): "Maven Pro Black",
                (2, 3, 1, 0x409): "Regular",
                (3, 3, 1, 0x409): "2.003;NONE;MavenPro-Black",
                (4, 3, 1, 0x409): "Maven Pro Black",
                (6, 3, 1, 0x409): "MavenPro-Black",
                (16, 3, 1, 0x409): "Maven Pro",
                (17, 3, 1, 0x409): "Black",
            },
        ),
        # Maven Pro Black Italic
        (
            mavenpro_fp,
            "Maven Pro",
            "Black Italic",
            [],
            {
                (1, 3, 1, 0x409): "Maven Pro Black",
                (2, 3, 1, 0x409): "Italic",
                (3, 3, 1, 0x409): "2.003;NONE;MavenPro-BlackItalic",
                (4, 3, 1, 0x409): "Maven Pro Black Italic",
                (6, 3, 1, 0x409): "MavenPro-BlackItalic",
                (16, 3, 1, 0x409): "Maven Pro",
                (17, 3, 1, 0x409): "Black Italic",
            },
        ),
        # Maven Pro ExtraLight Italic
        (
            mavenpro_fp,
            "Maven Pro",
            "ExtraLight Italic",
            [],
            {
                (1, 3, 1, 0x409): "Maven Pro ExtraLight",
                (2, 3, 1, 0x409): "Italic",
                (3, 3, 1, 0x409): "2.003;NONE;MavenPro-ExtraLightItalic",
                (4, 3, 1, 0x409): "Maven Pro ExtraLight Italic",
                (6, 3, 1, 0x409): "MavenPro-ExtraLightItalic",
                (16, 3, 1, 0x409): "Maven Pro",
                (17, 3, 1, 0x409): "ExtraLight Italic",
            },
        ),
        # check non-weight styles get appended to family name
        # Maven Pro UltraExpanded Regular
        (
            mavenpro_fp,
            "Maven Pro",
            "UltraExpanded Regular",
            [],
            {
                (1, 3, 1, 0x409): "Maven Pro UltraExpanded",
                (2, 3, 1, 0x409): "Regular",
                (3, 3, 1, 0x409): "2.003;NONE;MavenProUltraExpanded-Regular",
                (4, 3, 1, 0x409): "Maven Pro UltraExpanded Regular",
                (6, 3, 1, 0x409): "MavenProUltraExpanded-Regular",
                (16, 3, 1, 0x409): None,
                (17, 3, 1, 0x409): None,
            },
        ),
        # Maven Pro Condensed ExtraLight Italic
        (
            mavenpro_fp,
            "Maven Pro",
            "Condensed ExtraLight Italic",
            [],
            {
                (1, 3, 1, 0x409): "Maven Pro Condensed ExtraLight",
                (2, 3, 1, 0x409): "Italic",
                (3, 3, 1, 0x409): "2.003;NONE;MavenProCondensed-ExtraLightItalic",
                (4, 3, 1, 0x409): "Maven Pro Condensed ExtraLight Italic",
                (6, 3, 1, 0x409): "MavenProCondensed-ExtraLightItalic",
                (16, 3, 1, 0x409): "Maven Pro Condensed",
                (17, 3, 1, 0x409): "ExtraLight Italic",
            },
        ),
        ## VFs
        # Open Sans Roman
        (
            opensans_roman_fp,
            "Open Sans",
            None,
            [opensans_italic_fp, opensans_cond_roman_fp, opensans_cond_italic_fp],
            {
                (1, 3, 1, 0x409): "Open Sans",
                (2, 3, 1, 0x409): "Regular",
                (3, 3, 1, 0x409): "3.000;GOOG;OpenSans-Regular",
                (4, 3, 1, 0x409): "Open Sans Regular",
                (6, 3, 1, 0x409): "OpenSans-Regular",
                (16, 3, 1, 0x409): None,
                (17, 3, 1, 0x409): None,
                (25, 3, 1, 0x409): "OpenSans",
            },
        ),
        # Open Sans Italic
        (
            opensans_italic_fp,
            "Open Sans",
            None,
            [opensans_roman_fp, opensans_cond_roman_fp, opensans_cond_italic_fp],
            {
                (1, 3, 1, 0x409): "Open Sans",
                (2, 3, 1, 0x409): "Italic",
                (3, 3, 1, 0x409): "3.000;GOOG;OpenSans-Italic",
                (4, 3, 1, 0x409): "Open Sans Italic",
                (6, 3, 1, 0x409): "OpenSans-Italic",
                (16, 3, 1, 0x409): None,
                (17, 3, 1, 0x409): None,
                (25, 3, 1, 0x409): "OpenSansItalic",
            },
        ),
        # Open Sans Cond Roman
        (
            opensans_cond_roman_fp,
            "Open Sans Condensed",
            None,
            [opensans_roman_fp, opensans_italic_fp, opensans_cond_italic_fp],
            {
                (1, 3, 1, 0x409): "Open Sans Condensed",
                (2, 3, 1, 0x409): "Regular",
                (3, 3, 1, 0x409): "3.000;GOOG;OpenSansCondensed-Regular",
                (4, 3, 1, 0x409): "Open Sans Condensed Regular",
                (6, 3, 1, 0x409): "OpenSansCondensed-Regular",
                (16, 3, 1, 0x409): None,
                (17, 3, 1, 0x409): None,
                (25, 3, 1, 0x409): "OpenSansCondensed",
            },
        ),
        # Open Sans Cond Italic
        (
            opensans_cond_italic_fp,
            "Open Sans Condensed",
            None,
            [opensans_roman_fp, opensans_italic_fp, opensans_cond_roman_fp],
            {
                (1, 3, 1, 0x409): "Open Sans Condensed",
                (2, 3, 1, 0x409): "Italic",
                (3, 3, 1, 0x409): "3.000;GOOG;OpenSansCondensed-Italic",
                (4, 3, 1, 0x409): "Open Sans Condensed Italic",
                (6, 3, 1, 0x409): "OpenSansCondensed-Italic",
                (16, 3, 1, 0x409): None,
                (17, 3, 1, 0x409): None,
                (25, 3, 1, 0x409): "OpenSansCondensedItalic",
            },
        ),
        # Bad names
        (
            mavenpro_fp,
            "Maven Pro",
            "Fat",  # this should get appended to the family name
            [],
            {
                (1, 3, 1, 0x409): "Maven Pro Fat",
                (2, 3, 1, 0x409): "Regular",
                (3, 3, 1, 0x409): "2.003;NONE;MavenProFat-Regular",
                (4, 3, 1, 0x409): "Maven Pro Fat Regular",
                (6, 3, 1, 0x409): "MavenProFat-Regular",
                (16, 3, 1, 0x409): None,
                (17, 3, 1, 0x409): None,
            },
        ),
        (
            wonky_fp,
            "Wonky",  # name exists in axis reg!
            None,
            [],
            {
                (1, 3, 1, 0x409): "Wonky",
                (2, 3, 1, 0x409): "Regular",
                (3, 3, 1, 0x409): "3.000;GOOG;Wonky-Regular",
                (4, 3, 1, 0x409): "Wonky Regular",
                (6, 3, 1, 0x409): "Wonky-Regular",
                (16, 3, 1, 0x409): None,
                (17, 3, 1, 0x409): None,
            },
        ),
    ],
)
def test_name_table(fp, family_name, style_name, siblings, expected):
    font = TTFont(fp)
    siblings = [TTFont(fp) for fp in siblings]
    build_name_table(font, family_name, style_name, siblings)
    _test_names(font, expected)


@pytest.mark.parametrize(
    "font_fp, dflt_coords, expected",
    [
        # Condensed variants
        (
            opensans_cond_roman_fp,
            {},
            [
                ("Thin", {"wght": 100.0}),
                ("ExtraLight", {"wght": 200.0}),
                ("Light", {"wght": 300.0}),
                ("Regular", {"wght": 400.0}),
                ("Medium", {"wght": 500.0}),
                ("SemiBold", {"wght": 600.0}),
                ("Bold", {"wght": 700.0}),
                ("ExtraBold", {"wght": 800.0}),
            ],
        ),
        (
            opensans_cond_italic_fp,
            {},
            [
                ("Thin Italic", {"wght": 100.0}),
                ("ExtraLight Italic", {"wght": 200.0}),
                ("Light Italic", {"wght": 300.0}),
                ("Italic", {"wght": 400.0}),
                ("Medium Italic", {"wght": 500.0}),
                ("SemiBold Italic", {"wght": 600.0}),
                ("Bold Italic", {"wght": 700.0}),
                ("ExtraBold Italic", {"wght": 800.0}),
            ],
        ),
        # Multi axis + roman & ital with dflt coords
        (
            roboto_flex_fp,
            {},
            [
                ("Thin", {"wght": 100.0}),
                ("ExtraLight", {"wght": 200.0}),
                ("Light", {"wght": 300.0}),
                ("Regular", {"wght": 400.0}),
                ("Medium", {"wght": 500.0}),
                ("SemiBold", {"wght": 600.0}),
                ("Bold", {"wght": 700.0}),
                ("ExtraBold", {"wght": 800.0}),
                ("Black", {"wght": 900.0}),
                ("Thin Italic", {"wght": 100.0, "slnt": -10}),
                ("ExtraLight Italic", {"wght": 200.0, "slnt": -10}),
                ("Light Italic", {"wght": 300.0, "slnt": -10}),
                ("Italic", {"wght": 400.0, "slnt": -10}),
                ("Medium Italic", {"wght": 500.0, "slnt": -10}),
                ("SemiBold Italic", {"wght": 600.0, "slnt": -10}),
                ("Bold Italic", {"wght": 700.0, "slnt": -10}),
                ("ExtraBold Italic", {"wght": 800.0, "slnt": -10}),
                ("Black Italic", {"wght": 900.0, "slnt": -10}),
            ],
        ),
        # multi axis with METADATA.pb registry_default_overrides
        # https://github.com/google/fonts/blob/main/ofl/robotoflex/METADATA.pb
        (
            roboto_flex_fp,
            {
                "XOPQ": 96.0,
                "XTRA": 468.0,
                "YOPQ": 79.0,
                "YTDE": -203.0,
                "YTFI": 738.0,
                "YTLC": 514.0,
                "YTUC": 712.0,
            },
            [
                ("Thin", {"wght": 100.0}),
                ("ExtraLight", {"wght": 200.0}),
                ("Light", {"wght": 300.0}),
                ("Regular", {"wght": 400.0}),
                ("Medium", {"wght": 500.0}),
                ("SemiBold", {"wght": 600.0}),
                ("Bold", {"wght": 700.0}),
                ("ExtraBold", {"wght": 800.0}),
                ("Black", {"wght": 900.0}),
                ("Thin Italic", {"wght": 100.0, "slnt": -10}),
                ("ExtraLight Italic", {"wght": 200.0, "slnt": -10}),
                ("Light Italic", {"wght": 300.0, "slnt": -10}),
                ("Italic", {"wght": 400.0, "slnt": -10}),
                ("Medium Italic", {"wght": 500.0, "slnt": -10}),
                ("SemiBold Italic", {"wght": 600.0, "slnt": -10}),
                ("Bold Italic", {"wght": 700.0, "slnt": -10}),
                ("ExtraBold Italic", {"wght": 800.0, "slnt": -10}),
                ("Black Italic", {"wght": 900.0, "slnt": -10}),
            ],
        ),
    ],
)
def test_fvar_instances(font_fp, dflt_coords, expected):
    font = TTFont(font_fp)
    builder = build_fvar_instances(font, dflt_coords)
    fvar = font["fvar"]
    wght_axis = next((a for a in fvar.axes if a.axisTag == "wght"), None)
    wght_min = wght_axis.minValue
    wght_max = wght_axis.maxValue
    instances = fvar.instances
    if not dflt_coords:
        dflt_coords = {k: v["value"] for k, v in _fvar_dflts(font).items()}

    assert len(instances) == len(expected)
    for inst, (expected_name, coords) in zip(instances, expected):
        inst_name = font["name"].getName(inst.subfamilyNameID, 3, 1, 0x409).toUnicode()
        assert inst_name == expected_name
        for tag, val in coords.items():
            dflt_coords[tag] = val
        assert inst.coordinates == dflt_coords


def dump(table, ttFont=None):
    return "\n".join(getXML(table.toXML, ttFont))


@pytest.mark.parametrize(
    "fp, sibling_fps",
    [
        (roboto_flex_fp, []),
        (
            opensans_roman_fp,
            [opensans_italic_fp, opensans_cond_roman_fp, opensans_cond_italic_fp],
        ),
        (
            opensans_italic_fp,
            [opensans_roman_fp, opensans_cond_roman_fp, opensans_cond_italic_fp],
        ),
        (
            opensans_cond_roman_fp,
            [opensans_roman_fp, opensans_italic_fp, opensans_cond_italic_fp],
        ),
        (
            opensans_cond_italic_fp,
            [opensans_roman_fp, opensans_italic_fp, opensans_cond_roman_fp],
        ),
        (wonky_fp, []),
    ],
)
def test_stat(fp, sibling_fps):
    font = TTFont(fp)
    siblings = [TTFont(f) for f in sibling_fps]
    build_stat(font, siblings)
    stat_fp = fp.replace(".ttf", "_STAT.ttx")

    ### output good files
    # with open(stat_fp, "w") as doc:
    #    got = dump(font["STAT"], font)
    #    doc.write(got)

    with open(stat_fp) as doc:
        expected = doc.read()
        got = dump(font["STAT"], font)
        assert got == expected


@pytest.mark.parametrize(
    "fp, expected",
    [
        (mavenpro_fp, "MavenPro-Regular.ttf"),
        (opensans_roman_fp, "OpenSans[wdth,wght].ttf"),
        (opensans_italic_fp, "OpenSans-Italic[wdth,wght].ttf"),
        (opensans_cond_roman_fp, "OpenSansCondensed[wght].ttf"),
        (opensans_cond_italic_fp, "OpenSansCondensed-Italic[wght].ttf"),
    ],
)
def test_filename(fp, expected):
    font = TTFont(fp)
    assert build_filename(font) == expected


@pytest.mark.parametrize(
    "fp, sibling_fps, result",
    [
        (roboto_flex_fp, [], False),
        (
            opensans_roman_fp,
            [opensans_italic_fp, opensans_cond_roman_fp, opensans_cond_italic_fp],
            True,
        ),
        (
            opensans_italic_fp,
            [opensans_roman_fp, opensans_cond_roman_fp, opensans_cond_italic_fp],
            True,
        ),
        (
            opensans_cond_roman_fp,
            [opensans_roman_fp, opensans_italic_fp, opensans_cond_italic_fp],
            True,
        ),
        (
            opensans_cond_italic_fp,
            [opensans_roman_fp, opensans_italic_fp, opensans_cond_roman_fp],
            True,
        ),
        (opensans_roman_fp, [opensans_cond_roman_fp], True),
        (opensans_roman_fp, [opensans_italic_fp], False),
        (wonky_fp, [], False),
    ],
)
def test_fvar_instance_collisions(fp, sibling_fps, result):
    ttFont = TTFont(fp)
    siblings = [TTFont(fp) for fp in sibling_fps]
    assert _fvar_instance_collisions(ttFont, siblings) == result


@pytest.mark.parametrize(
    "fp, result",
    [
        (roboto_flex_fp, "RobotoFlex"),
        (opensans_roman_fp, "OpenSans"),
        (opensans_italic_fp, "OpenSansItalic"),
        (opensans_cond_roman_fp, "OpenSansCondensed"),
        (opensans_cond_italic_fp, "OpenSansCondensedItalic"),
        (wonky_fp, "Wonky"),
    ],
)
def test_build_variations_ps_name(fp, result):
    ttFont = TTFont(fp)
    build_variations_ps_name(ttFont)
    variation_ps_name = ttFont["name"].getName(25, 3, 1, 0x409).toUnicode()
    assert variation_ps_name == result



"""
Regenerate ttx dumps. Run this is the axis registry files in
Lib/axisregistry/data/*.textproto have been updated.
"""
from fontTools.ttLib import TTFont
from fontTools.misc.testTools import getXML, parseXML
from glob import glob
import os


def dump(table, ttFont=None):
    return "\n".join(getXML(table.toXML, ttFont))


CWD = os.path.dirname(__file__)
DATA_DIR = os.path.join(CWD, "data")
font_fps = glob(os.path.join(DATA_DIR, "*.ttf"))

# Dump STATs
for fp in font_fps:
    font = TTFont(fp)
    try:
        stat = dump(font["STAT"], font)
        stat_fp = os.path.join(
            DATA_DIR, font.reader.file.name.replace(".ttf", "_STAT.ttx")
        )
        with open(stat_fp, "w") as doc:
            doc.write(stat)
    except:
        all



# coding: utf-8
#
# Copyright 2022 Google LLC All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# See AUTHORS.txt for the list of Authors and LICENSE.txt for the License.
from setuptools import setup

# Read the contents of the README file
with open("README.md") as f:
    long_description = f.read()

setup(
    name="axisregistry",
    use_scm_version={"write_to": "Lib/axisregistry/_version.py"},
    url="https://github.com/googlefonts/axisregistry/",
    description="A python API to access data from the Google Fonts variable fonts axis registry.",
    long_description=long_description,
    long_description_content_type="text/markdown",  # This is important!
    author=("Felipe Sanches"),
    author_email="juca@members.fsf.org",
    package_dir={"": "Lib"},
    packages=["axisregistry"],
    package_data={"axisregistry": ["data/*.textproto"]},
    zip_safe=False,
    classifiers=[
        "Intended Audience :: Developers",
        "Topic :: Text Processing :: Fonts",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
    ],
    python_requires=">=3.7",
    setup_requires=["setuptools_scm>=4,<6.1"],
    install_requires=["protobuf==3.19.4", "fonttools"],
)



from axisregistry import AxisRegistry


def test_AxisRegistry():
    registry = AxisRegistry()
    assert "GRAD" in registry.keys()
    for axis_tag in registry.keys():
        assert len(axis_tag) == 4
        for f in range(len(registry[axis_tag].fallback)):
            fallback = registry[axis_tag].fallback[f]

            # fallback names should not be space-separated.
            # (see: https://github.com/googlefonts/axisregistry/issues/7)
            assert " " not in fallback.name
